import java.io.*;
import java.util.*;

/* Judge Solution by Jesus Ramos, Exchange Rates Division A
 * This problem is trickier than it seems in terms of writing a program to solve it.
 * The first thing to do is build a graph of nodes with dual edges that contain the exchange rates
 * when queried you start from the source ItemA and attempt to follow a path to ItemB by traversing through this graph
 * The simplest solution is to use a Depth First Search to simply follow all paths through the graph, which won't be many,
 * but this can also be solved using Floyd-Warshall's Algorithm which is a dynamic programming approach to this kind of
 * graph traversal. With Floyd-Warshall you can build new rates very easily rather than searching for them every time.
 * In this program every time a new connection is found it is simply added to the graph of rates
 */

public class ExchangeRates
{
	static int[][] rates = new int[60][60]; // dual edge graph for exchange rates
	static String[] itemNames = new String[60]; // the items name for referencing indexes in the graph
	static int lastItem = 0; // which index to place the next new item
	static int rate1, rate2; // will be used in the DFS to calculate exchange rates
	static boolean[] seen = new boolean[60]; // prevents visting the same path twice
	
	public static void main(String[] args) throws Exception
	{
		Scanner in = new Scanner(System.in);
		//PrintWriter out = new PrintWriter("exchange.out");
		while (in.hasNextLine())
		{
			String query = in.nextLine();
			if (query.equals("."))
				break;
			Scanner queryParse = new Scanner(query);
			String action = queryParse.next();
			if (action.equals("!"))
			{
				int r1 = queryParse.nextInt();
				String item1 = queryParse.next();
				queryParse.next(); // ignore the equals sign
				int r2 = queryParse.nextInt();
				String item2 = queryParse.next();
				int ind1 = findItem(item1);
				int ind2 = findItem(item2);
				// keep the conversion factors in lowest terms always
				int div = greatestCommonFactor(r1, r2);
				r1 /= div;
				r2 /= div;
				// if the item does not exist yet add create a new index for it
				if (ind1 == -1)
				{
					ind1 = lastItem;
					itemNames[lastItem++] = item1;
				}
				if (ind2 == -1)
				{
					ind2 = lastItem;
					itemNames[lastItem++] = item2;
				}
				// using these indexes build the dual edge from a to b and b to a
				rates[ind1][ind2] = r1;
				rates[ind2][ind1] = r2;
			}
			else
			{
				String item1 = queryParse.next();
				queryParse.next(); // ignore the equals sign
				String item2 = queryParse.next();
				int i1 = findItem(item1);
				int i2 = findItem(item2);
				// set rates to 1 so you can multiply the rates at each step in the DFS
				rate1 = 1;
				rate2 = 1;
				Arrays.fill(seen, false); // reset the array for each new traversal
				if (i1 !=-1 && i2 != -1 && convert(i1, i2)) // check that both components are in the graph first
				{
					// now that you know the conversion rate you can build a new dual edge so you don't have to search for it anymore
					rates[i1][i2] = rate1;
					rates[i2][i1] = rate2;
					System.out.println(rate1 + " " + item1 + " = " + rate2 + " " + item2);
				}
				else // no conversion factor exists yet
					System.out.println("? " + item1 + " = ? " + item2);
			}
		}
		//out.close();
	}
	// do DFS through the graph to find a conversion factor if any exists, this is the bulk of the algorithm
	static boolean convert(int i1, int i2)
	{
		seen[i1] = true; // mark each one you go to as visited
		if (rates[i1][i2] != 0) // if that part of the graph is already connected why search?
		{
			rate1 *= rates[i1][i2];
			rate2 *= rates[i2][i1];
			int div = greatestCommonFactor(rate1,rate2);
			rate1 /= div;
			rate2 /= div;
			return true;
		}
		if (i1 == i2) // you've arrived at destination
			return true;
		for (int i = 0; i < lastItem; i++)
			if (!seen[i] && rates[i1][i] != 0)
			{
				rate1 *= rates[i1][i];
				rate2 *= rates[i][i1];
				// keep all the numbers from overflowing
				int div = greatestCommonFactor(rate1,rate2);
				rate1 /= div;
				rate2 /= div;
				if (convert(i, i2))
					return true;
			}
		return false;
	}
	static int findItem(String s) // find the index of the string in the items array
	{
		for (int i = 0; i < 60; i++)
			if (itemNames[i] != null && itemNames[i].equals(s))
				return i;
		return -1;
	}
	/* This method is simply used to reduce the end result of a query into simplest terms
	 * this is Euler's greatest common factor method which was unwound from the recursive definition
	 * for reference the definition is :
	 * gcd(a,b) - if (b == 0) return a; else return gcd(b, a%b);
	 */
	static int greatestCommonFactor(int a, int b)
	{
		while (true)
		{
			if (b == 0)
				return a;
			int temp = a;
			a = b;
			b = temp % b;
		}
	}
}